using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Linq;
using System.Threading.Tasks;
using InternshipSystem.Core;
using InternshipSystem.Repository.Model;

namespace InternshipSystem.Repository
{
    public class DatabaseFiller
    {
        public DatabaseFiller(InternshipDbContext context)
        {
            Context = context;
        }

        public InternshipDbContext Context { get; }

        public async Task FillCompany()
        {
            var companies = new List<CompanyReadModel>
            {
                new CompanyReadModel
                {
                    Name = "Intel",
                    SiteAddress = new Uri("https://www.intel.com/content/www/us/en/jobs/locations/poland.html"),
                    Nip = "9570752316",
                    Range = RangeOfActivity.International,
                    Branches = new List<BranchOfficeReadModel>
                    {
                        new BranchOfficeReadModel
                        {
                            City = "Gdańsk",
                            Street = "ul. Słowackiego",
                            Building = "173",
                            PostalCode = "80-298",
                            Country = "Poland",
                        },
                        new BranchOfficeReadModel
                        {
                            City = "Gdańsk",
                            Street = "Jana z Kolna",
                            Building = "11",
                            PostalCode = "80-001",
                            Country = "Poland",
                        },
                        new BranchOfficeReadModel
                        {
                            City = "Boston",
                            Street = "State St",
                            Building = "53",
                            PostalCode = "MA 02109",
                            Country = "Stany Zjednoczone",
                        }
                    }
                },
                new CompanyReadModel
                {
                    Name = "Asseco Poland",
                    SiteAddress = new Uri("http://pl.asseco.com"),
                    Nip = "5842068320",
                    Range = RangeOfActivity.National,
                    Branches = new List<BranchOfficeReadModel>
                    {
                        new BranchOfficeReadModel
                        {
                            City = "Gdańsk",
                            Street = "ul. Podolska",
                            Building = "21",
                            PostalCode = "81-321",
                            Country = "Poland"
                        },
                        new BranchOfficeReadModel
                        {
                            City = "Wrocław",
                            Street = "Traugutta",
                            Building = "1/7",
                            PostalCode = "50-449",
                            Country = "Poland"
                        }
                    }
                }
            };
            await Context.Companies.AddRangeAsync(companies);
            await Context.SaveChangesAsync();
        } 

        public async Task FillStudents() 
        {
            var interns = new List<Student>
            {
                new Student {
                    FirstName = "Jan",
                    LastName = "Kowalski",
                    AlbumNumber = 123456,
                    Email = "s123456@student.pg.edu.pl",
                    Semester = 4,
                },
                new Student {
                    FirstName = "Adam",
                    LastName = "Kołek",
                    AlbumNumber = 102137,
                    Email = "s102137@student.pg.edu.pl",
                    Semester = 6,
                }
            };
            await Context.Students.AddRangeAsync(interns);
            await Context.SaveChangesAsync();
        }

        public async Task FillInternshipTypes() {
            var internshipTypes = new List<InternshipTypeReadModel>
            {
                new InternshipTypeReadModel 
                {
                    Type = "FreeInternship",
                    Description = "Praktyka bezpłatna",
                },
                new InternshipTypeReadModel 
                {
                    Type = "GraduateInternship"
                },
                new InternshipTypeReadModel 
                {
                    Type = "FreeApprenticeship"
                },
                new InternshipTypeReadModel
                {
                    Type = "PaidApprenticeship",
                    Description = "np. przemysłowy"
                },
                new InternshipTypeReadModel
                {
                    Type = "ForeignInternship",
                    Description = "np. IAESTE, ERASMUS"
                },
                new InternshipTypeReadModel
                {
                    Type = "UOP"
                },
                new InternshipTypeReadModel
                {
                    Type = "UD"
                },
                new InternshipTypeReadModel
                {
                    Type = "UZ"
                },
                new InternshipTypeReadModel
                {
                    Type = "Other",
                    Description = "Należy wprowadzić samodzielnie"
                }
            };
            await Context.InternshipTypes.AddRangeAsync(internshipTypes);
            await Context.SaveChangesAsync();
        }

        public async Task FillEditions() {
            var editions = new List<EditionReadModel>
            {
                new EditionReadModel 
                {
                    EditionStart = new DateTime(2000, 5, 10),
                    EditionFinish = new DateTime(2000, 11, 10),
                    ReportingStart = new DateTime(2000, 9, 30),
                    AvailableSubjects = new List<EditionSubject>
                    {
                        new EditionSubject 
                        {
                            InternshipId = Context.InternshipTypes
                                .First(i => i.Description == "Instalacja, konfiguracja i administracja niewielkich sieci komputerowych, w tym bezprzewodowych.")
                                .Id,
                        },
                        new EditionSubject
                        {
                            InternshipId = Context.InternshipTypes
                                .First(i => i.Description == "Projektowanie, implementacja i modyfikacjeoprogramowaniaw różnych technologiach i dla różnych zastosowań.")
                                .Id
                        }
                    }.ToImmutableList(),
                    Course = new Course
                    {
                        Name = "Informatyka",
                    }
                }
            };
            await Context.Editions.AddRangeAsync(editions);
            await Context.SaveChangesAsync();
        }

        public async Task FillInternShips() {
            var edition = Context.Editions.First();

            var intenrships = new List<InternshipReadModel>
            {
                new InternshipReadModel
                {
                    Student = Context.Students.First(i => i.AlbumNumber == 123456),
                    InternshipRegistration = new InternshipRegistrationReadModel()
                    {
                        Company = Context.Companies.First(c => c.Nip == "9570752316"), //Intel
                        Type = Context.InternshipTypes.First(t => t.Type == "UOP"),
                        Start = new DateTime(2000, 7, 1),
                        End = new DateTime(2000, 8, 30),
                        State = DocumentState.Submitted,
                        BranchAddress = 
                            Context.Companies
                                .First(c => c.Nip == "9570752316")
                                .Branches
                                .First(),
                    },
                    InternshipProgram = new InternshipProgramReadModel()
                    {
                        FirstName = "Horacy",
                        LastName = "Wościcki",
                        Email = "howos@intel.com",
                        PhoneNumber = "605-555-555",
                        ChosenSubjects = new List<ProgramSubject>
                        {
                            new ProgramSubject
                            {
                                //TODO
                            }
                        }
                    },
                },
                new InternshipReadModel
                {
                    Student = Context.Students.First(i => i.AlbumNumber == 102137),
                    InternshipRegistration = new InternshipRegistrationReadModel()
                    {
                        Company = Context.Companies.First(c => c.Nip == "5842068320"), //Asseco
                        Type = Context.InternshipTypes.First(t => t.Type == "UZ"),
                        Start = new DateTime(2000, 7, 1),
                        End = new DateTime(2000, 8, 30),
                        State = DocumentState.Submitted,
                        BranchAddress = 
                            Context.Companies
                                .First(c => c.Nip == "5842068320")
                                .Branches
                                .First(),
                    },
                    InternshipProgram = new InternshipProgramReadModel()
                    {
                        FirstName = "Henryk",
                        LastName = "Polaciński",
                        Email = "hepol@asseco.pl",
                        PhoneNumber = "555-525-545",
                        ChosenSubjects = new List<ProgramSubject>
                        {
                            //TODO
                        },
                    },
                },
            };
            Context.Internships.AddRange(intenrships);
            await Context.SaveChangesAsync();
        }
    } 
    
    // new Internship
    // {
    // Intern = Context.Interns.Where(i => i.AlbumNumber == "102137").First(),
    // Edition = edition,
    // BranchOffice = 
    // Context.Companies
    //     .Where(c => c.Nip == "5842068320")      //Asseco
    // .First()
    //     .Branches
    //     .First(),
    // Type = Context.InternshipTypes.Where(t => t.Type == "UZ").First(),
    // StartDate = new DateTime(2000, 7, 1),
    // EndDate = new DateTime(2000, 8, 30),
    // IsAccepted = false,
    // DeanAcceptance = new DeanAcceptance
    // {
    //     IsDeansAcceptanceRequired = false
    // },
    // Insurance = new Insurance
    // {
    //     InsuranceDeliveryDate = new DateTime(2000, 6, 29),
    //     IsInsuranceRequired = true
    // }, 
    // LengthInWeeks = 9,
    // Grade = 0,
    // Program = new List<InternshipProgramEntry>
    // {
    //     edition.Subjects.First()
    // },
    // Mentor = new Mentor
    // {
    //     Name = "Horacy",
    //     Surname = "Wościcki",
    //     Email = "howos@intel.com",
    //     Phone = "605-555-555"
    // }
    // }
    
    // Course = new Course
    // {
    // Name = "Informatyka",
    // DesiredSemesters = new List<int> { 4, 6 },
    // ProgramEntries = new List<InternshipProgramEntry>
    // {
    //     new InternshipProgramEntry
    //     {
    //         Description = "Instalacja, konfiguracja i administracja niewielkich sieci komputerowych, w tym bezprzewodowych."
    //     },
    //     new InternshipProgramEntry
    //     {
    //         Description = "Implementacja polityki bezpieczeństwa informacji w firmie lub instytucji, instalacja ochrony antywirusowej, konfiguracja zapór ogniowych."
    //     },
    //     new InternshipProgramEntry
    //     {
    //         Description = "Instalacja, konfiguracja i administracja oprogramowania, w szczególnościsystemów operacyjnychiserwerów aplikacji."
    //     },
    //     new InternshipProgramEntry
    //     {
    //         Description = "Projektowanie, implementacja i modyfikacjeoprogramowaniaw różnych technologiach i dla różnych zastosowań."
    //     }
    // }
    // }
    
    // Course = new Course 
    // {
    // Name = "Robotyka",
    // DesiredSemesters = new List<int> { 6 },
    // ProgramEntries = new List<InternshipProgramEntry>
    // {
    //     new InternshipProgramEntry
    //     {
    //         Description = "Testowanie oprogramowania, także z wykorzystaniem narzędzi do testowania automatycznego."
    //     },
    //     new InternshipProgramEntry
    //     {
    //         Description = "Wykorzystanie otwartych komponentów programowych z uwzględnieniem prawnych zależności pomiędzy nimi a produktem wynikowym."
    //     },
    //     new InternshipProgramEntry
    //     {
    //         Description = "Projektowanie i implementacja baz danych oraz badanie ich wydajności."
    //     },
    //     new InternshipProgramEntry
    //     {
    //         Description = "Projektowanie i prototypowaniezaawansowanychinterfejsów użytkownika."
    //     }
    // }
    // }
    
    // new Internship
    // {
    //     Intern = Context.Interns.Where(i => i.AlbumNumber == "123456").First(),
    //     Edition = edition,
    //     BranchOffice = 
    //         Context.Companies
    //             .Where(c => c.Nip == "9570752316")      //Intel
    //             .First()
    //             .Branches
    //             .First(),
    //     Type = Context.InternshipTypes.Where(t => t.Type == "UOP").First(),
    //     StartDate = new DateTime(2000, 7, 1),
    //     EndDate = new DateTime(2000, 8, 30),
    //     IsAccepted = false,
    //     DeanAcceptance = new DeanAcceptance
    //     {
    //         AcceptanceDate = new DateTime(2000, 6, 26),
    //         IsDeansAcceptanceRequired = true,
    //         Reason = DeanAcceptanceReason.Semester
    //     },
    //     Insurance = new Insurance
    //     {
    //         InsuranceDeliveryDate = new DateTime(2000, 6, 29),
    //         IsInsuranceRequired = true
    //     }, 
    //     LengthInWeeks = 9,
    //     Grade = 0,
    //     Program = new List<InternshipProgramEntry>
    //     {
    //         edition.Subjects.First()
    //     },
    //     Mentor = new Mentor
    //     {
    //         Name = "Horacy",
    //         Surname = "Wościcki",
    //         Email = "howos@intel.com",
    //         Phone = "605-555-555"
    //     }
    // },
}